package models

import (
	"encoding/json"
	"fmt"
	"net/http"
	"strconv"
	"strings"

	"forge.grandlyon.com/apoyen/elections/internal/auth"
)

func (d *DataHandler) handleDeskRound(w http.ResponseWriter, r *http.Request) {
	id, _ := strconv.Atoi(strings.TrimPrefix(r.URL.Path, "/api/DeskRound/"))
	switch method := r.Method; method {
	case "GET":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN", "CAPTURER", "VISUALIZER":
			d.getDeskRound(w, r, id)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	case "PUT":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN":
			d.putDeskRound(w, r, id)
		case "CAPTURER", "VISUALIZER":
			http.Error(w, ErrorNotAuthorizeMethodOnRessource, http.StatusMethodNotAllowed)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	default:
		http.Error(w, "method not allowed", 400)
	}
}

func (d *DataHandler) getDeskRound(w http.ResponseWriter, r *http.Request, id int) {
	if id != 0 {
		var o DeskRound
		if err := d.db.Preload("Votes").Preload("Capturers").First(&o, id).Error; err != nil {
			http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
			return
		}
		json.NewEncoder(w).Encode(o)
	} else {
		var o []DeskRound
		d.db.Preload("Votes").Preload("Capturers").Find(&o)
		json.NewEncoder(w).Encode(o)
	}
}

func (d *DataHandler) putDeskRound(w http.ResponseWriter, r *http.Request, id int) {
	var o DeskRound
	fmt.Println(id)
	if err := d.db.Preload("Votes").Preload("Capturers").First(&o, id).Error; err != nil {
		http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
		return
	}

	var deskRound DeskRound
	err := json.NewDecoder(r.Body).Decode(&deskRound)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}
	o.Validated = deskRound.Validated
	d.db.Save(&o)
	json.NewEncoder(w).Encode(o)
}

func (d *DataHandler) createDeskRound(roundID uint, deskID uint) {
	var o DeskRound
	o.RoundID = roundID
	o.DeskID = deskID
	o.Completed = false
	o.Validated = false

	d.db.Create(&o)
}

func (d *DataHandler) deleteDeskRound(deskRound DeskRound) {
	d.db.Delete(&deskRound)
}
